#!/usr/bin/python

import gtk
import pango
import searchentry
import control

global theme
theme = '''
style "notebook"
{
    bg[NORMAL] = @bg_color
    bg[ACTIVE] = @bg_color
    engine "murrine"
    {
        roundness = 0
    }
}
style "separator"
{
    engine "murrine"
    {
        separatorstyle = 0
    }
}
style "base"
{
    bg[NORMAL] = @base_color
}

class "GtkNotebook" style "notebook"
class "GtkSeparator" style "separator"
widget "*info_box*" style "base"
'''

class Gui:
    def __init__(self):
        '''Setting up the UI'''
        control.__init__()
        self.ui = gtk.Builder()
        self.ui.add_from_file(control.controller.UI_PATH)
        self.window = self.ui.get_object("window")
        self.window.set_position(gtk.WIN_POS_CENTER)
        self.window.connect("destroy", gtk.main_quit)
        self.window.set_default_size(600, 400)
        self.window.set_title("Lubuntu Software Center")
        self.window.set_name("LSCMainWindow")
        #-------------------------------------------------------
        '''Setting up left pane'''
        self.box7 = self.ui.get_object("vbox7")
        self.status_model = gtk.ListStore(str, str, int)
        self.status_view = gtk.TreeView(self.status_model)
        self.status_view.set_headers_visible(False)
        self.status_column = gtk.TreeViewColumn()
        self.status_icon_cell = gtk.CellRendererPixbuf()
        self.status_label_cell = gtk.CellRendererText()
        self.status_column.pack_start(self.status_icon_cell, False)
        self.status_column.pack_start(self.status_label_cell)
        self.status_column.add_attribute(self.status_icon_cell, "icon-name", 1)
        self.status_column.add_attribute(self.status_icon_cell, "stock-size", 2)
        self.status_column.add_attribute(self.status_label_cell, "text", 0)
        for item in (["Get Software", "stock_down", 3], ["Installed Software", "harddrive", 3], ["Apps Basket", "applications-other", 3]):
            self.status_model.append([item[0], item[1], item[2]])
        self.status_view.append_column(self.status_column)
        self.box7.pack_start(self.status_view)
        self.status_view.set_cursor(0)
        self.status_view.set_name("LSCStatusView")
        #-------------------------------------------------------
        '''Setting up the categories view'''
        self.categories_model = gtk.ListStore(str, 'GdkPixbuf', str, str)
        self.categories = self.ui.get_object("categories_view")
        self.categories.set_model(self.categories_model)
        self.categories_dict = {}
        #-------------------------------------------------------
        '''Setting up the available applications view'''
        self.apps_view = self.ui.get_object("apps_view")
        self.apps_scrolled = self.ui.get_object("scrolledwindow1")
        self.apps_column = self.ui.get_object("apps_column")
        self.apps_model = gtk.ListStore(str, str, int, str, str, int)
        self.apps_view.set_model(self.apps_model)
        self.apps_cell = gtk.CellRendererText()
        self.apps_type_cell = gtk.CellRendererPixbuf()
        self.apps_status_cell = gtk.CellRendererPixbuf()
        self.apps_column.pack_start(self.apps_type_cell, False)
        self.apps_column.pack_start(self.apps_cell)
        self.apps_column.pack_start(self.apps_status_cell, False)
        self.apps_column.add_attribute(self.apps_cell, "markup", 0)
        self.apps_column.add_attribute(self.apps_type_cell, "icon-name", 1)
        self.apps_column.add_attribute(self.apps_type_cell, "stock-size", 2)
        self.apps_column.add_attribute(self.apps_status_cell, "stock-id", 4)
        self.apps_column.add_attribute(self.apps_status_cell, "stock-size", 5)
        self.categorie_icon = self.ui.get_object("categorie_icon")
        self.categorie_label = self.ui.get_object("categorie_label")
        self.pkgs_count = self.ui.get_object("pkgs_count")
        #-------------------------------------------------------
        '''Setting up the installed applications view'''
        self.installed_view = self.ui.get_object("installed_view")
        self.installed_column = self.ui.get_object("installed_column")
        self.installed_model = gtk.ListStore(str, str, int, str, str, int)
        self.installed_view.set_model(self.installed_model)
        self.installed_cell = gtk.CellRendererText()
        self.installed_type_cell = gtk.CellRendererPixbuf()
        self.installed_status_cell = gtk.CellRendererPixbuf()
        self.installed_column.pack_start(self.installed_type_cell, False)
        self.installed_column.pack_start(self.installed_cell)
        self.installed_column.pack_end(self.installed_status_cell, False)
        self.installed_column.add_attribute(self.installed_cell, "markup", 0)
        self.installed_column.add_attribute(self.installed_type_cell, "icon-name", 1)
        self.installed_column.add_attribute(self.installed_type_cell, "stock-size", 2)
        self.installed_column.add_attribute(self.installed_status_cell, "stock-id", 4)
        self.installed_column.add_attribute(self.installed_status_cell, "stock-size", 5)
        self.installed_count = self.ui.get_object("installed_count")
        self.installed_label = self.ui.get_object("installed_label")
        #-------------------------------------------------------
        '''Setting up misc'''
        self.search_pkg = searchentry.Entry()
        self.search_pkg.set_sensitive(False)
        self.pages = self.ui.get_object("notebook1")
        self.pkgs_count = self.ui.get_object("pkgs_count")
        self.home = self.ui.get_object("home_button")
        self.toolbar = self.ui.get_object("toolbar1")
        self.toolbar.insert(self.search_pkg, 2)
        self.available = self.ui.get_object("available_button")
        self.installed = self.ui.get_object("installed_button")
        self.apps_basket = self.ui.get_object("apps_basket_button")
        self.preferences_button = self.ui.get_object("preferences_button")
        self.exit_menu = self.ui.get_object("exit_item")
        self.settings_item = self.ui.get_object("settings_item")
        self.sp_item = self.ui.get_object("sp_item")
        self.progressbar = self.ui.get_object("progress")
        self.progress_image = self.ui.get_object("image_progress")
        self.no_found_box = self.ui.get_object("no_found_box")
        self.no_found_button = self.ui.get_object("no_found_button")
        #-------------------------------------------------------
        '''Setting up available message box'''
        self.apps_box = self.ui.get_object("vbox4")
        self.close_image = gtk.Image()
        self.close_image.set_from_stock('gtk-close', 1)
        self.message_close = gtk.Button()
        self.message_close.set_tooltip_text("Hide this message")
        self.message_close.set_relief(gtk.RELIEF_NONE)
        self.message_close.set_image(self.close_image)
        self.message_label = gtk.Label()
        self.message_label.set_ellipsize(pango.ELLIPSIZE_END)
        self.message_more_info = gtk.Button("Details")
        self.message_add_tab = gtk.Button("Add to the Apps Basket")
        self.apps_message = gtk.InfoBar()
        self.apps_message_cont = self.apps_message.get_content_area()
        self.apps_message_cont.pack_start(self.message_close, False)
        self.apps_message_cont.pack_start(self.message_label)
        self.apps_message_cont.pack_start(self.message_more_info, False)
        self.apps_message_cont.pack_start(self.message_add_tab, False)
        self.apps_box.pack_start(self.apps_message, False)
        #-------------------------------------------------------
        '''Setting up installed message box'''
        self.installed_box = self.ui.get_object("installed_box")
        self.installed_message = gtk.InfoBar()
        self.installed_message_cont = self.installed_message.get_content_area()
        self.close_image2 = gtk.Image()
        self.close_image2.set_from_stock(gtk.STOCK_CLOSE, 1)
        self.message_inst_close = gtk.Button()
        self.message_inst_close.set_tooltip_text("Hide this message")
        self.message_inst_close.set_relief(gtk.RELIEF_NONE)
        self.message_inst_close.set_image(self.close_image2)
        self.message_inst_label = gtk.Label()
        self.message_inst_label.set_ellipsize(pango.ELLIPSIZE_END)
        self.message_inst_more_info = gtk.Button("Details")
        self.message_inst_remove = gtk.Button("Remove from the system")
        self.installed_message_cont.pack_start(self.message_inst_close, False)
        self.installed_message_cont.pack_start(self.message_inst_label)
        self.installed_message_cont.pack_start(self.message_inst_more_info, False)
        self.installed_message_cont.pack_start(self.message_inst_remove, False)
        self.installed_box.pack_start(self.installed_message, False)
        #-------------------------------------------------------
        self.marked_install_model = gtk.ListStore(str, str, str)
        self.marked_install_view = self.ui.get_object("marked_install_view")
        self.marked_install_view.set_model(self.marked_install_model)
        self.marked_install_view_column1 = self.ui.get_object("marked_install_column1")
        self.marked_install_view_column2 = self.ui.get_object("marked_install_column2")
        self.marked_install_view_column3 = self.ui.get_object("marked_install_column3")
        self.marked_install_cell1 = gtk.CellRendererText()
        self.marked_install_cell2 = gtk.CellRendererText()
        self.marked_install_cell3 = gtk.CellRendererText()
        self.marked_install_view_column1.pack_start(self.marked_install_cell1)
        self.marked_install_view_column2.pack_start(self.marked_install_cell2)
        self.marked_install_view_column3.pack_start(self.marked_install_cell3)
        self.marked_install_view_column1.add_attribute(self.marked_install_cell1, "text", 0)
        self.marked_install_view_column2.add_attribute(self.marked_install_cell2, "text", 1)
        self.marked_install_view_column3.add_attribute(self.marked_install_cell3, "text", 2)
        #-------------------------------------------------------
        self.infobox = self.ui.get_object("vbox1")
        self.pkgicon = self.ui.get_object("pkgicon")
        self.pkgtitle = self.ui.get_object("pkgtitle")
        self.pkgshortdesc = self.ui.get_object("pkgshortdesc")
        self.pkgdescription = self.ui.get_object("pkgdescription")
        self.pkgsize1 = self.ui.get_object("download_label")
        self.pkgsize2 = self.ui.get_object("inst_label")
        self.pkgversion = self.ui.get_object("version_label")
        self.back_button = self.ui.get_object("back_button")
        self.install_remove_button = self.ui.get_object("remove_install_button")
        self.port2 = self.ui.get_object("viewport2")
        self.port2.set_name("info_box")
        self.paned = self.ui.get_object("hpaned1")
        self.paned.set_name("LSCHPaned")
        #-------------------------------------------------------
        self.remove_mai_button = self.ui.get_object("remove_mai_button")
        self.riepilogue_label = self.ui.get_object("riepilogue_label")
        self.install_pkgs = self.ui.get_object("install_pkgs")
        self.marked_count_label = self.ui.get_object("marked_count")
        #-------------------------------------------------------
        gtk.rc_parse_string(theme)
        self.window.show_all()
        self.apps_message.hide()
        self.installed_message.hide()
        self.install_pkgs.set_sensitive(False)
        self.no_found_box.set_visible(False)
    
    def refresh(self, icon_theme):
        control.controller.append_categories(self, self.categories_model, icon_theme)
