#!/usr/bin/python

from ConfigParser import RawConfigParser
import gtk
import os

home = os.getenv("HOME")
default_conf_file = '''
[General]
ui_path = data/lsc.ui
preferences_ui = data/preferences.ui
infos_ui = data/infos.ui
app_install_directory = /usr/share/app-install/desktop/
expert_mode = FALSE
categories_file = data/categories
categories_file_expert = data/categories_expert
'''


if not os.path.exists("/".join([home, ".LSC.ini"])):
    print("Creating new conf file in %s" % home)
    file = open("/".join([home, ".LSC.ini"]), "w")
    file.write(default_conf_file)
    file.close()

class Controller:
    def __init__(self):
        print("Opening config file")
        self.home = os.getenv("HOME")
        self.file_path = "/".join([self.home, ".LSC.ini"])
        self.conf_file = open(self.file_path, "r+w")
        self.parser = RawConfigParser()
        self.parser.readfp(self.conf_file)
        self.UI_PATH = self.parser.get("General", "UI_PATH")
        self.PREFERENCES_UI = self.parser.get("General", "PREFERENCES_UI")
        self.INFOS_UI = self.parser.get("General", "INFOS_UI")
        self.app_install_directory = self.parser.get("General", "app_install_directory")
        self.expert_mode = self.parser.getboolean("General", "expert_mode")
        self.categories_file = open(self.parser.get("General", "categories_file"), "r")

    def append_categories(self, ui, model, theme):
        model.clear()
        ui.categories_dict = {}
        if not self.expert_mode:
            for lines in self.categories_file:
                string = lines.split("||")
                ui.categories_dict[string[0]] = [string[1], string[2]]
            for items in ui.categories_dict:
                if items != "All":
                    icon = theme.lookup_icon(ui.categories_dict[items][0], 5, 0)
                    filename = icon.get_filename()
                    pixbuf = gtk.gdk.pixbuf_new_from_file(filename.replace("16", "32").replace("24", "32"))
                    model.append([items, pixbuf, ui.categories_dict[items][0], ui.categories_dict[items][1]])
            icon = theme.lookup_icon("distributor-logo", 5, 0)
            filename = icon.get_filename()
            pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(filename, 32, 32)
            model.append(["All", pixbuf, "distributor-logo", "all"])
        else:
            for lines in self.categories_file:
                string = lines.split("||")
                ui.categories_dict[string[0]] = [string[1], string[3].replace("\n", "")]
            for items in ui.categories_dict:
                if items != "All":
                    icon = theme.lookup_icon(ui.categories_dict[items][0], 5, 0)
                    filename = icon.get_filename()
                    pixbuf = gtk.gdk.pixbuf_new_from_file(filename.replace("16", "32").replace("24", "32"))
                    model.append([items, pixbuf, ui.categories_dict[items][0], ui.categories_dict[items][1]])
            icon = theme.lookup_icon("distributor-logo", 5, 0)
            filename = icon.get_filename()
            pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(filename, 32, 32)
            model.append(["All", pixbuf, "distributor-logo", "all"])
        self.categories_file.close()

def __init__():
    global controller
    controller = Controller()
