#!/usr/bin/python
# -*- coding:UTF-8 -*-
#
#  lubuntu-software-center.py
#
#    Stephen Smally <stephen.smally@gmail.com>
#
#       This program is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 2 of the License, or
#       (at your option) any later version.
#       
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#       
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.
#       

import gtk
import apt_pkg
import os
from LSC import UI, control, preferences, threadingops
import subprocess
import gettext

_ = gettext.gettext

class LscControl:
    '''The class that control the program'''
    def __init__(self):
        self.ui = UI.Gui()
        self.refresh_system_call()
        #-------------------------------------------------------
        self.ui.categories.connect("item-activated", self.on_selected_category)
        self.ui.home.connect("clicked", self.back_home)
        self.ui.preferences_button.connect("clicked", self.on_show_preferences)
        self.ui.status_view.connect("cursor-changed", self.on_change_status)
        self.ui.apps_view.connect("cursor-changed", self.on_selected_available)
        self.ui.apps_view.connect("row-activated", self.app_item_activated, 0)
        self.ui.installed_view.connect("cursor-changed", self.on_selected_installed)
        self.ui.installed_view.connect("row-activated", self.app_item_activated, 1)
        self.ui.message_close.connect("clicked", self.back_home, False)
        self.ui.message_add_tab.connect("clicked", self.on_add_tbi)
        self.ui.message_more_info.connect("clicked", self.on_more_info, 0)
        self.ui.message_inst_close.connect("clicked", lambda w: self.ui.installed_message.hide())
        self.ui.message_inst_more_info.connect("clicked", self.on_more_info, 1)
        self.ui.message_inst_remove.connect("clicked", self.remove_package)
        self.ui.marked_install_view.connect("cursor-changed", self.on_selected_marked)
        self.ui.remove_mai_button.connect("clicked", self.on_remove_marked)
        self.ui.install_pkgs.connect("clicked", self.on_install_pkgs)
        self.ui.exit_menu.connect("activate", gtk.main_quit)
        self.ui.settings_item.connect("activate", self.on_show_preferences)
        self.ui.sp_item.connect("activate", self.open_software_properties)
        self.ui.no_found_button.connect("clicked", self.search_in_all)
        self.ui.search_pkg.search_function = self.search_from_bar
        #------------------------------------------------------
    
    ###########################################################
    # Functions related to the "Available" section
    #    
    def search_in_all(self, widget):
        self.ui.no_found_box.set_visible(False)
        self.ui.apps_scrolled.set_visible(True)
        self.ui.apps_view.set_model(self.ui.apps_model)
        self.actual_category = "All"
        self.section = self.ui.categories_dict[self.actual_category]
        self.append_packages_call("all", [], self.ui.apps_model)
    
    def search_apps(self, string):
        self.search(1, string)
    
    def on_selected_category(self, widget, path):
        '''Handle the change of section'''
        self.choosed_category = self.ui.categories_model[path[0]]
        self.ui.pages.set_page(1)
        self.ui.categorie_icon.set_from_icon_name(self.choosed_category[2], 6)
        self.ui.categorie_label.set_text(self.choosed_category[0])
        self.actual_category = self.choosed_category[0]
        self.append_packages_call(self.choosed_category[3], [], self.ui.apps_model)
    
    def on_selected_available(self, widget):
        '''Show the bottom box with the message'''
        if self.ui.apps_view.get_selection().get_selected()[1] != None:
            self.pkg = self.current_apps_model[self.ui.apps_view.get_selection().get_selected()[1]]
            self.pkg_selected = (self.pkg[3], self.pkg[0], self.pkg[1], self.pkg[4])
            self.ui.message_label.set_text(_("'%s' package selected") % self.pkg_selected[0])
            if not self.pkg_selected[3] == gtk.STOCK_YES:
                if self.pkg_selected[0] in self.marked_as_install:
                    self.ui.install_remove_button.set_label(_("Added to the Apps Basket"))
                    self.ui.install_remove_button.set_sensitive(False)
                    self.ui.message_add_tab.child.set_text(_("Added to the Apps Basket"))
                    self.ui.message_add_tab.set_sensitive(False)
                else:
                    self.ui.install_remove_button.set_label(_("Add to the Apps Basket"))
                    self.ui.install_remove_button.set_sensitive(True)
                    self.ui.message_add_tab.child.set_text(_("Add to the Apps Basket"))
                    self.ui.message_add_tab.set_sensitive(True)
                    self.ui.install_remove_button.connect("clicked", self.on_add_tbi_from_dialog)
                self.ui.install_remove_button.connect("clicked", self.on_add_tbi)
                self.ui.apps_message.set_visible(True)
            else:
                self.ui.message_add_tab.child.set_text(_("Remove from the system"))
                self.ui.message_add_tab.connect("clicked", self.remove_package)
                self.ui.install_remove_button.set_label(_("Remove from the system"))
                self.ui.install_remove_button.connect("clicked", self.remove_package)
                self.ui.apps_message.set_visible(True)
        else: pass
    
    ###########################################################
    # Functions related to the "Installed" section
    #
    def search_installed(self, string):
        self.search(2, string)
    
    def on_selected_installed(self, widget):
        '''Show the bottom box with the message'''
        try:
            self.pkg = self.current_installed_model[self.ui.installed_view.get_selection().get_selected()[1]]
            self.pkg_selected = (self.pkg[3], self.pkg[0], self.pkg[1])
            self.ui.message_inst_label.set_text("'%s' package selected" % self.pkg_selected[0])
            self.ui.install_remove_button.set_label("Remove from the system")
            self.ui.install_remove_button.set_sensitive(True)
            self.ui.installed_message.show()
            self.ui.install_remove_button.connect("clicked", self.remove_package)
        except TypeError:
            pass

    def remove_package(self, widget):
        self.response = subprocess.check_output(["python", "scripts/apc.py", "remove", self.pkg_selected[0]])
        if self.response == "success":
            self.refresh_system_call
    
    ###########################################################
    # Functions related to "Available" and "Installed" section
    #
    def define_icon(self, icon):
        if icon != None:
            if self.theme.has_icon(icon):
                return icon
            else:
                return "deb"
        else:
            return "deb"
        
    def append_packages_call(self, category="all", status=[apt_pkg.CURSTATE_NOT_INSTALLED], model=None):
        '''Append the packages in the given section to the given model, default non-installed'''
        self.packages.sort()
        model.clear()
        self.ui.paned.set_sensitive(False)
        self.call = threadingops.append_packages(self, category, status, model)
        self.call.start()
    
    def search(self, page, string):
        if string != "":
            if page == 2:
                self.installed_model_search = gtk.ListStore(str, str, int, str, str, int)
                for items in self.ui.installed_model:
                    if string.lower() in items[0].lower(): # lower() => no case-sensitive
                        self.installed_model_search.append([items[0], items[1], items[2], items[3], items[4], items[5]])
                self.ui.installed_view.set_model(self.installed_model_search)
                self.ui.installed_label.set_text("Searching in Installed")
                self.ui.installed_count.set_text("%s packages matching" % len(self.installed_model_search))
                self.current_installed_model = self.installed_model_search
            else:
                self.section = self.ui.categories_dict[self.actual_category]
                self.apps_model_search = gtk.ListStore(str, str, int, str, str, int)
                for items in self.ui.apps_model:
                    if string.lower() in items[0].lower(): # lower() => no case-sensitive
                        self.apps_model_search.append([items[0], items[1], items[2], items[3], items[4], items[5]])
                self.ui.apps_view.set_model(self.apps_model_search)
                self.ui.categorie_icon.set_from_icon_name(self.section[0], 6)
                self.ui.categorie_label.set_text("Searching in %s" % self.actual_category)
                self.ui.pkgs_count.set_text("%s packages matching" % len(self.apps_model_search))
                self.current_apps_model = self.apps_model_search
                if len(self.current_apps_model) == 0:
                    if self.actual_category != "All":
                        self.ui.apps_scrolled.set_visible(False)
                        self.ui.no_found_box.set_visible(True)
                else:
                    self.ui.apps_scrolled.set_visible(True)
                    self.ui.no_found_box.set_visible(False)
        else:
            if page == 2:
                self.ui.installed_view.set_model(self.ui.installed_model)
                self.ui.installed_label.set_text("Installed")
                self.ui.installed_count.set_text("%s packages listed" % len(self.ui.installed_model))
                self.current_installed_model = self.ui.installed_model
            else:
                self.section = self.ui.categories_dict[self.actual_category]
                self.ui.apps_view.set_model(self.ui.apps_model)
                self.ui.categorie_icon.set_from_icon_name(self.section[0], 6)
                self.ui.categorie_label.set_text(self.actual_category)
                self.ui.pkgs_count.set_text("%s packages listed" % len(self.ui.apps_model))
                self.current_apps_model = self.ui.apps_model
    
    ###########################################################
    # Functions related to the "Apps Basket"
    #
    def refresh_app_basket(self):
        self.ui.remove_mai_button.hide()
        self.ui.marked_install_model.clear()
        self.total_download = 0
        self.depends_added = []
        for items in self.marked_as_install:
            self.infos_pkg = self.get_pkg_depends(items)
            if self.infos_pkg == False:
                self.marked_as_install = []
                self.refresh_app_basket()
                break
            self.total_download += self.infos_pkg[0]
            self.ui.marked_install_model.append([items, apt_pkg.size_to_str(self.infos_pkg[0]), self.infos_pkg[1]])
            for item in self.infos_pkg[2]:
                if self.cache[item[0][0]].current_state != apt_pkg.CURSTATE_INSTALLED:
                    self.infos_dep = self.get_pkg_depends(item[0][0])
                    if self.infos_dep == False:
                        self.marked_as_install = []
                        self.refresh_app_basket()
                        break
                    if not item in self.depends_added:
                        self.total_download += self.infos_dep[0] 
                    self.ui.marked_install_model.append([" "*4+self.infos_dep[3]+" (requested by %s)" % items, apt_pkg.size_to_str(self.infos_dep[0]), self.infos_dep[1]])
                    self.depends_added.append(item)
        if len(self.ui.marked_install_model) == 0:
            self.ui.marked_count_label.set_text("0 packages marked")
            self.ui.riepilogue_label.set_text("No package marked for installation")
            self.ui.install_pkgs.set_sensitive(False)
        else:
            self.ui.marked_count_label.set_text("%s packages marked" % len(self.ui.marked_install_model))
            self.ui.riepilogue_label.set_text("%s to download" % apt_pkg.size_to_str(self.total_download))
            self.ui.install_pkgs.set_sensitive(True)
        self.on_selected_available(None)
    
    def on_selected_marked(self, widget):
        self.selection = widget.get_model()[widget.get_selection().get_selected()[1]][0]
        if self.selection in self.marked_as_install:
            self.ui.remove_mai_button.set_label("Remove '%s' from the Apps Basket" % self.selection)
            self.ui.remove_mai_button.set_visible(True)
        else:
            self.ui.remove_mai_button.set_visible(False)
        self.on_selected_available(None)
    
    def on_remove_marked(self, widget):
        self.marked_as_install.remove(self.selection)
        self.refresh_app_basket()
    
    def get_pkg_depends(self, pkg):
        if not self.cache[pkg].version_list == []:
            self.version = self.depcache.GetCandidateVer(self.cache[pkg])
            try:
                self.depends = self.version.depends_list_str["Depends"]
            except KeyError:
                self.depends = []
            return (self.version.size, self.version.ver_str, self.depends, self.version.parent_pkg.name)
        elif self.cache[pkg].provides_list == []:
            self.error = gtk.MessageDialog(self.ui.window, 0, gtk.MESSAGE_ERROR, gtk.BUTTONS_CLOSE, "Error")
            self.error.format_secondary_text("The package %s has no candidate to install" % pkg)
            self.error.run()
            self.error.destroy()
            return False            
        else:
            self.version = self.cache[pkg].provides_list[0][-1]
            try:
                self.depends = self.version.depends_list_str["Depends"]
            except KeyError:
                self.depends = []
            return (self.version.size, self.version.ver_str, self.depends, self.version.parent_pkg.name)
    
    def on_install_pkgs(self, widget):
        self.marked_as_install_str = ",".join(self.marked_as_install)
        self.response = subprocess.check_output(["python", "scripts/apc.py", "install", self.marked_as_install_str])
        if self.response == "success":
            self.refresh_system_call
        
    ###########################################################
    # Functions related to all the Gui
    #
    def search_from_bar(self, search_string):
        self.search(self.choosed_page, search_string)
    
    def back_home(self, widget, back=True):
        '''Back home sweet home'''
        self.ui.apps_message.hide()
        if back:
            self.ui.status_view.set_cursor(0)
            self.ui.pages.set_page(0)
            self.ui.search_pkg.set_sensitive(False)
            self.choosed_page = 1
    
    def on_change_status(self, widget):
        '''Handle the change of status (Available/Installed)'''
        self.status_dict = {
        "Get Software": 1,
        "Installed Software": 2,
        "Apps Basket": 4
        }
        self.choosed_page = self.status_dict[self.ui.status_model[widget.get_selection().get_selected()[1]][0]]
        if self.choosed_page == 1: # if you choose the Available list
            if len(self.ui.apps_model) != 0:
                self.ui.pages.set_page(1)
            else:
                self.ui.pages.set_page(0)
                self.ui.search_pkg.set_sensitive(False)
        if self.choosed_page == 2: # if you choose the Installed list
            if len(self.ui.installed_model) == 0:
                self.append_packages_call("all", [
                apt_pkg.CURSTATE_NOT_INSTALLED,
                apt_pkg.CURSTATE_CONFIG_FILES,
                apt_pkg.CURSTATE_HALF_CONFIGURED,
                apt_pkg.CURSTATE_HALF_INSTALLED,
                apt_pkg.CURSTATE_UNPACKED
                ], self.ui.installed_model)
            self.ui.pages.set_page(2)
        if self.choosed_page == 4: # if you choose the Apps Basket
            self.refresh_app_basket()
            self.ui.pages.set_page(3)
            self.ui.search_pkg.set_sensitive(False)
    
    def on_reactive_window(self, widget):
        widget.hide()
        self.ui.window.set_sensitive(True)
    
    def on_add_tbi_from_dialog(self, widget):
        widget.set_label("Added to the Apps Basket")
        widget.set_sensitive(False)
        self.on_add_tbi(None)
    
    def on_more_info(self, widget, status):
        self.candidate = self.depcache.GetCandidateVer(self.cache[self.pkg_selected[0]])
        self.infos.Lookup(self.candidate.FileList[0])
        self.setup_infos(
        self.ui.pages,
        self.pkg_selected[2],
        "\n".join([self.pkg_selected[1].split("\n")[0], self.infos.short_desc]),
        self.infos.long_desc.replace("\n ", "\n"),
        (apt_pkg.size_to_str(self.candidate.size),
        apt_pkg.size_to_str(self.candidate.installed_size)),
        self.candidate.ver_str,
        self.ui.pages.get_current_page()
        )
    
    def app_item_activated(self, widget, path, column, inst_rem):
        self.pkg = widget.get_model()[path[0]]
        self.pkg_selected = (self.pkg[3], self.pkg[0], self.pkg[1])
        self.on_more_info(None, inst_rem)
    
    def on_add_tbi(self, widget):
        self.pkg_name = self.pkg_selected[0]
        if not self.pkg_name in self.marked_as_install:
            self.marked_as_install.append(self.pkg_name)
            self.on_selected_available(None)
    
    def on_show_preferences(self, widget):
        self.preferences_dialog = preferences.Preferences_UI()
        self.preferences_dialog.refresh_func = self.refresh_system_call
        self.preferences_dialog.show()
    
    def pop_error(self, message):
        self.dialog = gtk.MessageDialog(self.ui.window, 0, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, "Error")
        self.dialog.format_secondary_text(message)
        self.dialog.run()
        self.dialog.destroy()
    
    def open_software_properties(self, widget):
        subprocess.call(["gksu", "--desktop", "/usr/share/applications/software-properties-gtk.desktop", "/usr/bin/software-properties-gtk"])
    
    def setup_infos(self, pages, icon, title, description, size, version, last_page):
        self.ui.pkgicon.set_from_icon_name(icon, 6)
        self.ui.pkgtitle.set_text(title.split("\n")[0])
        self.ui.pkgshortdesc.set_text(title.split("\n")[-1])
        self.ui.pkgdescription.set_text(description.replace("\n.", "\n"))
        self.ui.pkgsize1.set_text("Download Size: %s" % size[0])
        self.ui.pkgsize2.set_text("Installed Size: %s" % size[1])
        self.ui.pkgversion.set_text("Version: %s" % version)
        self.ui.back_button.connect("clicked", lambda w: self.ui.pages.set_current_page(last_page))
        self.ui.pages.set_current_page(4)
    
    def define_packages(self, name, pkg, icon, categories, (cur, tot)):
        self.packages.append([name, pkg, icon, categories])
        self.ui.progressbar.set_fraction(float(cur)/float(tot))
    
    def refresh_system_call(self):
        self.ui.pages.set_page(0)
        apt_pkg.init()
        self.cache = apt_pkg.Cache()
        self.depcache = apt_pkg.DepCache(self.cache)
        self.infos = apt_pkg.PackageRecords(self.cache)
        control.__init__()
        self.aid = control.controller.app_install_directory
        self.marked_as_install = []
        self.theme = gtk.icon_theme_get_default()
        self.ui.apps_model.clear()
        self.ui.installed_model.clear()
        self.choosed_page = 1
        self.current_apps_model = self.ui.apps_model
        self.current_installed_model = self.ui.installed_model
        self.refresh_app_basket()
        self.ui.apps_view.set_model(self.ui.apps_model)
        self.ui.installed_view.set_model(self.ui.installed_model)
        self.ui.apps_message.set_visible(False)
        self.ui.installed_message.set_visible(False)
        self.ui.refresh(self.theme)
        self.packages = []
        self.refresh = threadingops.refresh_system(self)
        self.refresh.start()
    
    def append_packages_appending(self, items,  status, status_dict, model, (cur, tot)):
        self.pkg_in_cache = self.cache[items[1]]
        if not self.pkg_in_cache.current_state in status:
            self.candidate = self.depcache.GetCandidateVer(self.pkg_in_cache)
            if self.candidate != None:
                self.infos.Lookup(self.candidate.FileList[0])
                model.append(["\n".join([items[0], "<span size='small'>%s</span>" % self.infos.short_desc]), self.define_icon(items[2]), 5, items[1], status_dict[self.pkg_in_cache.current_state], 4])
                self.ui.progressbar.set_fraction(float(cur)/float(tot))

if __name__ == "__main__":
    global app
    app = LscControl()
    gtk.main()
